@@ -8,6 +8,7 @@ import os
 import time
 import sys
 
+from biomaj_core.utils import Utils
 from biomaj_core.bmajindex import BmajIndex
 
 if sys.version < '3':
@@ -40,8 +41,10 @@ class BiomajConfig(object):
         'db.type': '',
         'db.formats': '',
         'keep.old.version': 1,
+        'keep.old.sessions': 0,
         'docker.sudo': '1',
-        'auto_publish': 0
+        'auto_publish': 0,
+        'use_hardlinks': 0
     }
 
     # Old biomaj level compatibility
@@ -191,9 +194,11 @@ class BiomajConfig(object):
             if options is not None and options.get_option('log') is not None:
                 hdlr.setLevel(BiomajConfig.LOGLEVEL[options.get_option('log')])
                 self.log_level = BiomajConfig.LOGLEVEL[options.get_option('log')]
+                logger.setLevel(self.log_level)
             else:
                 hdlr.setLevel(BiomajConfig.LOGLEVEL[self.get('historic.logfile.level')])
                 self.log_level = BiomajConfig.LOGLEVEL[self.get('historic.logfile.level')]
+                logger.setLevel(self.log_level)
             formatter = logging.Formatter('%(asctime)s %(levelname)-5.5s [%(name)s][%(threadName)s] %(message)s')
             hdlr.setFormatter(formatter)
             logger.addHandler(hdlr)
@@ -281,19 +286,9 @@ class BiomajConfig(object):
         """
         Get a boolean property from bank or general configration. Optionally in section.
         """
-        value = None
+        value = self.get(prop, section, escape, default)
 
-        if self._in_env(prop):
-            value = self._in_env(prop)
-        else:
-            value = self.get(prop, section, escape, default)
-
-        if value is None:
-            return False
-        if value is True or value == 'true' or value == '1':
-            return True
-        else:
-            return False
+        return Utils.to_bool(value)
 
     def get(self, prop, section='GENERAL', escape=True, default=None):
         """
@@ -402,8 +397,8 @@ class BiomajConfig(object):
             status = False
         else:
             protocol = self.get('protocol')
-            allowed_protocols = ['none', 'multi', 'local', 'ftp', 'sftp', 'http',
-                                 'https', 'directftp', 'directhttp', 'directhttps', 'rsync', 'irods']
+            allowed_protocols = ['none', 'multi', 'local', 'ftp', 'ftps', 'http', 'https',
+                                 'directftp', 'directftps', 'directhttp', 'directhttps', 'rsync', 'irods']
             if protocol not in allowed_protocols:
                 logging.error('Protocol not supported: ' + protocol)
                 status = False
@@ -417,7 +412,7 @@ class BiomajConfig(object):
                 elif not self.get('remote.dir').endswith('/'):
                     logging.error('remote.dir must end with a /')
                     return False
-                if protocol not in ['direcftp', 'directhttp', 'directhttps'] and\
+                if protocol not in ['directftp', 'directftps', 'directhttp', 'directhttps'] and\
                         not self.get('remote.files') and\
                         not self.get('remote.list'):
                     logging.error('remote.files not set')
