@@ -1,4 +1,5 @@
 import os
+import errno
 import re
 import logging
 import shutil
@@ -152,17 +153,18 @@ class Utils(object):
 
         Each file is a dict like with (at least) parameters: year, month, day
         """
-        release = None
+        if not files:
+            return None
+        # release = None
+        rfile = files[0]
+        release = {'year': rfile['year'], 'month': rfile['month'], 'day': rfile['day']}
         for rfile in files:
-            if release is None:
-                release = {'year': rfile['year'], 'month': rfile['month'], 'day': rfile['day']}
-            else:
-                rel_date = datetime.date(int(release['year']), int(release['month']), int(release['day']))
-                file_date = datetime.date(int(rfile['year']), int(rfile['month']), int(rfile['day']))
-                if file_date > rel_date:
-                    release['year'] = rfile['year']
-                    release['month'] = rfile['month']
-                    release['day'] = rfile['day']
+            rel_date = datetime.date(int(release['year']), int(release['month']), int(release['day']))
+            file_date = datetime.date(int(rfile['year']), int(rfile['month']), int(rfile['day']))
+            if file_date > rel_date:
+                release['year'] = rfile['year']
+                release['month'] = rfile['month']
+                release['day'] = rfile['day']
         return release
 
     @staticmethod
@@ -195,7 +197,8 @@ class Utils(object):
             }[date]
 
     @staticmethod
-    def copy_files(files_to_copy, to_dir, move=False, lock=None):
+    def copy_files(files_to_copy, to_dir, move=False, lock=None,
+                   use_hardlinks=False):
         """
         Copy or move files to to_dir, keeping directory structure.
 
@@ -214,6 +217,8 @@ class Utils(object):
         :type move: bool
         :param lock: thread lock object for multi-threads
         :type lock: Lock
+        :param use_hardlinks: use hard links (if possible)
+        :type link: bool
         """
         logger = logging.getLogger('biomaj')
         nb_files = len(files_to_copy)
@@ -244,14 +249,48 @@ class Utils(object):
             else:
                 start_time = datetime.datetime.now()
                 start_time = time.mktime(start_time.timetuple())
-                shutil.copyfile(from_file, to_file)
+                if use_hardlinks:
+                    try:
+                        os.link(from_file, to_file)
+                        logger.debug("Using hardlinks to copy %s",
+                                     file_to_copy['name'])
+                    except OSError as e:
+                        if e.errno in (errno.ENOSYS, errno.ENOTSUP):
+                            msg = "Your system doesn't support hard links. Using regular copy."
+                            logger.warn(msg)
+                            # Copy this file (the stats are copied at the end
+                            # of the function)
+                            shutil.copyfile(from_file, to_file)
+                            # Don't try links anymore
+                            use_hardlinks = False
+                        elif e.errno == errno.EPERM:
+                            msg = "The FS at %s doesn't support hard links. Using regular copy."
+                            logger.warn(msg, to_dir)
+                            # Copy this file (the stats are copied at the end
+                            # of the function)
+                            shutil.copyfile(from_file, to_file)
+                            # Don't try links anymore
+                            use_hardlinks = False
+                        elif e.errno == errno.EXDEV:
+                            msg = "Cross device hard link is impossible (source: %s, dest: %s). Using regular copy."
+                            logger.warn(msg, from_file, to_dir)
+                            # Copy this file
+                            shutil.copyfile(from_file, to_file)
+                            # Don't try links anymore
+                            use_hardlinks = False
+                        else:
+                            raise
+                else:
+                    shutil.copyfile(from_file, to_file)
                 end_time = datetime.datetime.now()
                 end_time = time.mktime(end_time.timetuple())
                 file_to_copy['download_time'] = end_time - start_time
-                shutil.copystat(from_file, to_file)
+                if not use_hardlinks:
+                    shutil.copystat(from_file, to_file)
 
     @staticmethod
-    def copy_files_with_regexp(from_dir, to_dir, regexps, move=False, lock=None):
+    def copy_files_with_regexp(from_dir, to_dir, regexps, move=False, lock=None,
+                               use_hardlinks=False):
         """
         Copy or move files from from_dir to to_dir matching regexps.
         Copy keeps the original file stats.
@@ -266,20 +305,28 @@ class Utils(object):
         :type move: bool
         :param lock: thread lock object for multi-threads
         :type lock: Lock
+        :param use_hardlinks: use hard links (if possible)
+        :type link: bool
         :return: list of copied files with their size
         """
         logger = logging.getLogger('biomaj')
         files_to_copy = []
-        for root, dirs, files in os.walk(from_dir, topdown=True):
+        files_list = []
+        for root, _, files in os.walk(from_dir, topdown=True):
             for name in files:
                 for reg in regexps:
                     file_relative_path = os.path.join(root, name).replace(from_dir, '')
                     if file_relative_path.startswith('/'):
                         file_relative_path = file_relative_path.replace('/', '', 1)
+                    # sometimes files appear twice.... check not already managed
+                    if file_relative_path in files_list:
+                        continue
                     if reg == "**/*":
                         files_to_copy.append({'name': file_relative_path})
+                        files_list.append(file_relative_path)
                         continue
                     if re.match(reg, file_relative_path):
+                        files_list.append(file_relative_path)
                         files_to_copy.append({'name': file_relative_path})
                         continue
 
@@ -305,8 +352,43 @@ class Utils(object):
             if move:
                 shutil.move(from_file, to_file)
             else:
-                shutil.copyfile(from_file, to_file)
-                shutil.copystat(from_file, to_file)
+                if use_hardlinks:
+                    try:
+                        os.link(from_file, to_file)
+                        logger.debug("Using hardlinks to copy %s",
+                                     file_to_copy['name'])
+                    except OSError as e:
+                        if e.errno in (errno.ENOSYS, errno.ENOTSUP):
+                            msg = "Your system doesn't support hard links. Using regular copy."
+                            logger.warn(msg)
+                            # Copy this file (the stats are copied at the end
+                            # of the function)
+                            shutil.copyfile(from_file, to_file)
+                            # Don't try links anymore
+                            use_hardlinks = False
+                        elif e.errno == errno.EPERM:
+                            msg = "The FS at %s doesn't support hard links. Using regular copy."
+                            logger.warn(msg, to_dir)
+                            # Copy this file (we copy the stats here because
+                            # it's not done at the end of the function)
+                            shutil.copyfile(from_file, to_file)
+                            shutil.copystat(from_file, to_file)
+                            # Don't try links anymore
+                            use_hardlinks = False
+                        elif e.errno == errno.EXDEV:
+                            msg = "Cross device hard link is impossible (source: %s, dest: %s). Using regular copy."
+                            logger.warn(msg, from_file, to_dir)
+                            # Copy this file (we copy the stats here because
+                            # it's not done at the end of the function)
+                            shutil.copyfile(from_file, to_file)
+                            shutil.copystat(from_file, to_file)
+                            # Don't try links anymore
+                            use_hardlinks = False
+                        else:
+                            raise
+                else:
+                    shutil.copyfile(from_file, to_file)
+                    shutil.copystat(from_file, to_file)
             file_to_copy['size'] = os.path.getsize(to_file)
             f_stat = datetime.datetime.fromtimestamp(os.path.getmtime(to_file))
             file_to_copy['year'] = str(f_stat.year)
@@ -384,3 +466,28 @@ class Utils(object):
             os.remove(archivefile)
 
         return True
+
+    @staticmethod
+    def to_bool(value):
+        if isinstance(value, bool):
+            return value
+        if not value:
+            return False
+        try:
+            if value.lower() == 'true' or value == '1':
+                return True
+            else:
+                return False
+        except Exception:
+            return False
+
+    @staticmethod
+    def to_int(value):
+        if isinstance(value, int):
+            return value
+        if not value:
+            return 0
+        try:
+            return int(value)
+        except Exception:
+            return 0
